@@ -16,15 +16,18 @@
 
 #include "TouchGestureArea.h"
 
-// local
-#include "TouchOwnershipEvent.h"
-#include "TouchRegistry.h"
-#include "UnownedTouchEvent.h"
+#include <UbuntuGestures/private/touchownershipevent_p.h>
+#include <UbuntuGestures/private/touchregistry_p.h>
+#include <UbuntuGestures/private/unownedtouchevent_p.h>
+// #include "TouchRegistry.h"
+// #include "UnownedTouchEvent.h"
 
 #include <QGuiApplication>
 #include <QStyleHints>
 #include <private/qquickwindow_p.h>
 
+UG_USE_NAMESPACE
+
 #define TOUCHGESTUREAREA_DEBUG 0
 
 // TODO - understand more about why we lose touch event releases.
@@ -134,7 +137,7 @@ TouchGestureArea::TouchGestureArea(QQuickItem* parent)
     , m_recognitionPeriod(50)
     , m_releaseRejectPeriod(100)
 {
-    setRecognitionTimer(new UbuntuGestures::Timer(this));
+    setRecognitionTimer(new Timer(this));
     m_recognitionTimer->setInterval(m_recognitionPeriod);
     m_recognitionTimer->setSingleShot(true);
 }
@@ -152,10 +155,10 @@ bool TouchGestureArea::event(QEvent *event)
 {
     // Process unowned touch events (handles update/release for incomplete gestures)
     if (event->type() == TouchOwnershipEvent::touchOwnershipEventType()) {
-        touchOwnershipEvent(static_cast<TouchOwnershipEvent *>(event));
+        touchOwnershipEvent(static_cast<TouchOwnershipEvent*>(event));
         return true;
     } else if (event->type() == UnownedTouchEvent::unownedTouchEventType()) {
-        unownedTouchEvent(static_cast<UnownedTouchEvent *>(event)->touchEvent());
+        unownedTouchEvent(static_cast<UnownedTouchEvent*>(event)->touchEvent());
         return true;
     }
 
@@ -374,7 +377,7 @@ void TouchGestureArea::unownedTouchEvent_waitingForMoreTouches(QTouchEvent *even
         }
     }
 
-    if (m_candidateTouches.count() == 0) {
+    if (m_candidateTouches.isEmpty()) {
         setInternalStatus(InternalStatus::WaitingForTouch);
     }
 }
@@ -438,7 +441,7 @@ void TouchGestureArea::unownedTouchEvent_rejected(QTouchEvent *event)
         }
     }
 
-    if (m_watchedTouches.count() == 0) {
+    if (m_watchedTouches.isEmpty()) {
         setInternalStatus(InternalStatus::WaitingForTouch);
     }
 }
@@ -526,7 +529,7 @@ void TouchGestureArea::updateTouchPoints(QTouchEvent *touchEvent)
     if (updateable) {
         if (!dragging() && m_status == InternalStatus::Recognized) {
             bool allWantDrag = !m_liveTouchPoints.isEmpty();
-            Q_FOREACH(auto point, m_liveTouchPoints) {
+            Q_FOREACH(const auto &point, m_liveTouchPoints) {
                 allWantDrag &= point->dragging();
             }
             // only dragging if all points are dragging.
@@ -588,13 +591,15 @@ void TouchGestureArea::setInternalStatus(uint newStatus)
             resyncCachedTouchPoints();
             break;
         case InternalStatus::WaitingForMoreTouches:
-            m_recognitionTimer->start(m_recognitionPeriod);
+            m_recognitionTimer->setInterval(m_recognitionPeriod);
+            m_recognitionTimer->start();
             break;
         case InternalStatus::Recognized:
             resyncCachedTouchPoints();
             break;
         case InternalStatus::WaitingForRejection:
-            m_recognitionTimer->start(m_releaseRejectPeriod);
+            m_recognitionTimer->setInterval(m_releaseRejectPeriod);
+            m_recognitionTimer->start();
             break;
         case InternalStatus::Rejected:
             resyncCachedTouchPoints();
@@ -605,7 +610,7 @@ void TouchGestureArea::setInternalStatus(uint newStatus)
     }
 }
 
-void TouchGestureArea::setRecognitionTimer(UbuntuGestures::AbstractTimer *timer)
+void TouchGestureArea::setRecognitionTimer(AbstractTimer *timer)
 {
     int interval = 0;
     bool timerWasRunning = false;
@@ -643,11 +648,9 @@ bool TouchGestureArea::dragging() const
 QQmlListProperty<GestureTouchPoint> TouchGestureArea::touchPoints()
 {
     return QQmlListProperty<GestureTouchPoint>(this,
-                                                    0,
-                                                    nullptr,
-                                                    TouchGestureArea::touchPoint_count,
-                                                    TouchGestureArea::touchPoint_at,
-                                               0);
+                                               nullptr,
+                                               TouchGestureArea::touchPoint_count,
+                                               TouchGestureArea::touchPoint_at);
 }
 
 int TouchGestureArea::minimumTouchPoints() const
@@ -718,7 +721,7 @@ void TouchGestureArea::rejectGesture()
     }
     m_candidateTouches.clear();
 
-    if (m_watchedTouches.count() == 0) {
+    if (m_watchedTouches.isEmpty()) {
         setInternalStatus(InternalStatus::WaitingForTouch);
     } else {
         setInternalStatus(InternalStatus::Rejected);
