@@ -30,6 +30,9 @@
 #include <QDateTime>
 #include <QDBusUnixFileDescriptor>
 
+// Glib
+#include <glib.h>
+
 #define LOGIN1_SERVICE QStringLiteral("org.freedesktop.login1")
 #define LOGIN1_PATH QStringLiteral("/org/freedesktop/login1")
 #define LOGIN1_IFACE QStringLiteral("org.freedesktop.login1.Manager")
@@ -61,19 +64,19 @@ public:
                                                           QStringLiteral("GetSessionByPID"));
         msg << (quint32) getpid();
 
-        QDBusReply<QDBusObjectPath> reply = QDBusConnection::systemBus().call(msg);
+        QDBusReply<QDBusObjectPath> reply = QDBusConnection::SM_BUSNAME().call(msg);
         if (reply.isValid()) {
             logindSessionPath = reply.value().path();
 
             // start watching the Active property
-            QDBusConnection::systemBus().connect(LOGIN1_SERVICE, logindSessionPath, QStringLiteral("org.freedesktop.DBus.Properties"), QStringLiteral("PropertiesChanged"),
-                                                 this, SLOT(onPropertiesChanged(QString,QVariantMap,QStringList)));
+            QDBusConnection::SM_BUSNAME().connect(LOGIN1_SERVICE, logindSessionPath, QStringLiteral("org.freedesktop.DBus.Properties"), QStringLiteral("PropertiesChanged"),
+                                                  this, SLOT(onPropertiesChanged(QString,QVariantMap,QStringList)));
 
             setupSystemdInhibition();
 
             // re-enable the inhibition upon resume from sleep
-            QDBusConnection::systemBus().connect(LOGIN1_SERVICE, LOGIN1_PATH, LOGIN1_IFACE, QStringLiteral("PrepareForSleep"),
-                                                 this, SLOT(onResuming(bool)));
+            QDBusConnection::SM_BUSNAME().connect(LOGIN1_SERVICE, LOGIN1_PATH, LOGIN1_IFACE, QStringLiteral("PrepareForSleep"),
+                                                  this, SLOT(onResuming(bool)));
         } else {
             qWarning() << "Failed to get logind session path" << reply.error().message();
         }
@@ -93,7 +96,7 @@ public:
         msg << "Unity8 handles power events"; // why
         msg << "block"; // mode
 
-        QDBusPendingCall pendingCall = QDBusConnection::systemBus().asyncCall(msg);
+        QDBusPendingCall pendingCall = QDBusConnection::SM_BUSNAME().asyncCall(msg);
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pendingCall, this);
         connect(watcher, &QDBusPendingCallWatcher::finished,
             this, [this](QDBusPendingCallWatcher* watcher) {
@@ -111,7 +114,7 @@ public:
     bool checkLogin1Call(const QString &method) const
     {
         QDBusMessage msg = QDBusMessage::createMethodCall(LOGIN1_SERVICE, LOGIN1_PATH, LOGIN1_IFACE, method);
-        QDBusReply<QString> reply = QDBusConnection::systemBus().call(msg);
+        QDBusReply<QString> reply = QDBusConnection::SM_BUSNAME().call(msg);
         return reply.isValid() && (reply == QStringLiteral("yes") || reply == QStringLiteral("challenge"));
     }
 
@@ -122,7 +125,7 @@ public:
                                                           LOGIN1_IFACE,
                                                           method);
         msg.setArguments(args);
-        QDBusConnection::systemBus().asyncCall(msg);
+        QDBusConnection::SM_BUSNAME().asyncCall(msg);
     }
 
     void setActive(bool active)
@@ -154,7 +157,7 @@ public:
         msg << LOGIN1_SESSION_IFACE;
         msg << ACTIVE_KEY;
 
-        QDBusPendingCall pendingCall = QDBusConnection::systemBus().asyncCall(msg);
+        QDBusPendingCall pendingCall = QDBusConnection::SM_BUSNAME().asyncCall(msg);
         QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pendingCall, this);
         connect(watcher, &QDBusPendingCallWatcher::finished,
             this, [this](QDBusPendingCallWatcher* watcher) {
@@ -188,7 +191,7 @@ public:
         msg << LOGIN1_SESSION_IFACE;
         msg << IDLE_SINCE_KEY;
 
-        QDBusReply<QVariant> reply = QDBusConnection::systemBus().call(msg);
+        QDBusReply<QVariant> reply = QDBusConnection::SM_BUSNAME().call(msg);
         if (reply.isValid()) {
             return reply.value().value<quint64>();
         } else {
@@ -205,7 +208,7 @@ public:
                                                           LOGIN1_SESSION_IFACE,
                                                           QStringLiteral("SetIdleHint"));
         msg << idle;
-        QDBusConnection::systemBus().asyncCall(msg);
+        QDBusConnection::SM_BUSNAME().asyncCall(msg);
     }
 
 private Q_SLOTS:
@@ -224,11 +227,14 @@ private Q_SLOTS:
     {
         if (!active) {
             setupSystemdInhibition();
+        } else {
+            Q_EMIT prepareForSleep();
         }
     }
 
 Q_SIGNALS:
     void screensaverActiveChanged(bool active);
+    void prepareForSleep();
 };
 
 Q_GLOBAL_STATIC(DBusUnitySessionServicePrivate, d)
@@ -237,11 +243,12 @@ DBusUnitySessionService::DBusUnitySessionService()
     : UnityDBusObject(QStringLiteral("/com/canonical/Unity/Session"), QStringLiteral("com.canonical.Unity"))
 {
     if (!d->logindSessionPath.isEmpty()) {
-        // connect our Lock() slot to the logind's session Lock() signal
-        QDBusConnection::systemBus().connect(LOGIN1_SERVICE, d->logindSessionPath, LOGIN1_SESSION_IFACE, QStringLiteral("Lock"), this, SLOT(Lock()));
+        // connect our PromptLock() slot to the logind's session Lock() signal
+        QDBusConnection::SM_BUSNAME().connect(LOGIN1_SERVICE, d->logindSessionPath, LOGIN1_SESSION_IFACE, QStringLiteral("Lock"), this, SLOT(PromptLock()));
         // ... and our Unlocked() signal to the logind's session Unlock() signal
         // (lightdm handles the unlocking by calling logind's Unlock method which in turn emits this signal we connect to)
-        QDBusConnection::systemBus().connect(LOGIN1_SERVICE, d->logindSessionPath, LOGIN1_SESSION_IFACE, QStringLiteral("Unlock"), this, SIGNAL(Unlocked()));
+        QDBusConnection::SM_BUSNAME().connect(LOGIN1_SERVICE, d->logindSessionPath, LOGIN1_SESSION_IFACE, QStringLiteral("Unlock"), this, SLOT(doUnlock()));
+        connect(d, &DBusUnitySessionServicePrivate::prepareForSleep, this, &DBusUnitySessionService::PromptLock);
     } else {
         qWarning() << "Failed to connect to logind's session Lock/Unlock signals";
     }
@@ -295,12 +302,7 @@ bool DBusUnitySessionService::CanLock() const
 
 QString DBusUnitySessionService::UserName() const
 {
-    struct passwd *p = getpwuid(geteuid());
-    if (p) {
-        return QString::fromUtf8(p->pw_name);
-    }
-
-    return QString();
+    return QString::fromUtf8(g_get_user_name());
 }
 
 QString DBusUnitySessionService::RealName() const
@@ -330,15 +332,41 @@ QString DBusUnitySessionService::HostName() const
 
 void DBusUnitySessionService::PromptLock()
 {
+    // Prompt as in quick.  No locking animation needed.  Usually used by
+    // indicator-session in combination with a switch to greeter or other
+    // user session.
     Q_EMIT LockRequested();
     Q_EMIT lockRequested();
 }
 
 void DBusUnitySessionService::Lock()
 {
-    // signal u8 to show the lockscreen/greeter
+    // Normal lock (with animation, as compared to PromptLock above).  Usually
+    // used by indicator-session to lock the session in place.
+    //
+    // FIXME: We also -- as a bit of a hack around indicator-session not fully
+    // supporting a phone profile -- switch to greeter here.  The unity7 flow is
+    // that the user chooses "Lock/Switch" from the indicator, and then can go
+    // to greeter by selecting "Switch" again from the indicator, which is now
+    // exposed by the desktop_lockscreen profile.  But since in unity8, we try
+    // to expose most things all the time, we don't use the separate lockscreen
+    // profile.  Instead, we just go directly to the greeter the first time
+    // a user presses "Lock/Switch".  This isn't what this DBus call is
+    // supposed to do, but we can live with it for now.
+    //
+    // Here's a bug about indicator-session growing a converged Touch profile:
+    // https://launchpad.net/bugs/1557716
+    //
+    // We only do this here in the animated-lock call because that's the only
+    // time the indicator locks without also asking the display manager to
+    // switch sessions on us.  And since we are switching screens, we also
+    // don't bother respecting the animate request, simply doing a PromptLock.
     PromptLock();
+    switchToGreeter();
+}
 
+void DBusUnitySessionService::switchToGreeter()
+{
     // lock the session using the org.freedesktop.DisplayManager system DBUS service
     const QString sessionPath = QString::fromLocal8Bit(qgetenv("XDG_SESSION_PATH"));
     QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.DisplayManager"),
@@ -346,7 +374,7 @@ void DBusUnitySessionService::Lock()
                                                       QStringLiteral("org.freedesktop.DisplayManager.Session"),
                                                       QStringLiteral("Lock"));
 
-    QDBusPendingCall pendingCall = QDBusConnection::systemBus().asyncCall(msg);
+    QDBusPendingCall pendingCall = QDBusConnection::SM_BUSNAME().asyncCall(msg);
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pendingCall, this);
     connect(watcher, &QDBusPendingCallWatcher::finished,
         this, [this](QDBusPendingCallWatcher* watcher) {
@@ -363,6 +391,12 @@ void DBusUnitySessionService::Lock()
     });
 }
 
+void DBusUnitySessionService::doUnlock()
+{
+    Q_EMIT Unlocked();
+    Q_EMIT unlocked();
+}
+
 bool DBusUnitySessionService::IsLocked() const
 {
     return !d->isSessionActive;
@@ -392,16 +426,19 @@ void DBusUnitySessionService::Shutdown()
 
 void DBusUnitySessionService::Suspend()
 {
+    PromptLock();
     d->makeLogin1Call(QStringLiteral("Suspend"), {false});
 }
 
 void DBusUnitySessionService::Hibernate()
 {
+    PromptLock();
     d->makeLogin1Call(QStringLiteral("Hibernate"), {false});
 }
 
 void DBusUnitySessionService::HybridSleep()
 {
+    PromptLock();
     d->makeLogin1Call(QStringLiteral("HybridSleep"), {false});
 }
 
@@ -481,7 +518,7 @@ void DBusGnomeScreensaverWrapper::SetActive(bool lock)
 
 void DBusGnomeScreensaverWrapper::Lock()
 {
-    performAsyncUnityCall(QStringLiteral("Lock"));
+    performAsyncUnityCall(QStringLiteral("PromptLock"));
 }
 
 quint32 DBusGnomeScreensaverWrapper::GetActiveTime() const
@@ -518,7 +555,7 @@ bool DBusScreensaverWrapper::SetActive(bool lock)
 
 void DBusScreensaverWrapper::Lock()
 {
-    performAsyncUnityCall(QStringLiteral("Lock"));
+    performAsyncUnityCall(QStringLiteral("PromptLock"));
 }
 
 quint32 DBusScreensaverWrapper::GetActiveTime() const
