@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Canonical, Ltd.
+ * Copyright (C) 2014-2016 Canonical, Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,6 +14,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <QDebug>
+#include <QTimer>
+
 #include "MockSystemImage.h"
 
 MockSystemImage::MockSystemImage(QObject *parent)
@@ -21,7 +24,27 @@ MockSystemImage::MockSystemImage(QObject *parent)
 {
 }
 
+void MockSystemImage::checkForUpdate()
+{
+    qDebug() << "Doing a fake system update check";
+}
+
+void MockSystemImage::applyUpdate()
+{
+    qDebug() << "Applying a fake system update";
+    setUpdateApplying(true);
+    QTimer::singleShot(3000, [this] {setUpdateApplying(false);});
+}
+
 void MockSystemImage::factoryReset()
 {
     Q_EMIT resettingDevice();
 }
+
+void MockSystemImage::setUpdateApplying(bool status)
+{
+    if (status != m_updateApplying) {
+        m_updateApplying = status;
+        Q_EMIT updateApplyingChanged();
+    }
+}
