@@ -20,6 +20,8 @@
 
 MockTelepathyHelper::MockTelepathyHelper(QObject *parent)
     : QObject(parent)
+    , m_ready(true)
+    , m_emergencyCallsAvailable(true)
 {
 }
 
@@ -33,3 +35,28 @@ void MockTelepathyHelper::registerChannelObserver(const QString& name)
 {
     Q_UNUSED(name);
 }
+
+bool MockTelepathyHelper::ready() const
+{
+    return m_ready;
+}
+
+void MockTelepathyHelper::setReady(bool value)
+{
+    if (m_ready != value) {
+        m_ready = value;
+        Q_EMIT readyChanged();
+    }
+}
+bool MockTelepathyHelper::emergencyCallsAvailable() const
+{
+    return m_emergencyCallsAvailable;
+}
+
+void MockTelepathyHelper::setEmergencyCallsAvailable(bool value)
+{
+    if (m_emergencyCallsAvailable != value) {
+        m_emergencyCallsAvailable = value;
+        Q_EMIT emergencyCallsAvailableChanged();
+    }
+}
