@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Canonical, Ltd.
+ * Copyright (C) 2015,2016 Canonical, Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,16 +22,34 @@
 
 #include <QDebug>
 
-VirtualKeyboard::VirtualKeyboard(QObject *parent)
+VirtualKeyboard::VirtualKeyboard()
     : MirSurface("input-method",
                      Mir::InputMethodType,
                      Mir::MinimizedState,
                      QUrl("qrc:///Unity/Application/vkb_portrait.png"),
-                     QUrl("qrc:///Unity/Application/VirtualKeyboard.qml"),
-                     parent)
+                     QUrl("qrc:///Unity/Application/VirtualKeyboard.qml"))
 {
 }
 
 VirtualKeyboard::~VirtualKeyboard()
 {
 }
+
+void VirtualKeyboard::updateInputBoundsAfterResize()
+{
+    int width = this->width();
+
+    int height;
+    if (this->width() > this->height()) {
+        // landscape
+        height = this->height() * 0.4;
+    } else {
+        // portrait
+        height = this->width() * 0.6;
+    }
+
+    int x = 0;
+    int y = this->height() - height;
+
+    setInputBounds(QRect(x, y, width, height));
+}
