@@ -34,6 +34,8 @@
 #include <QDBusMetaType>
 #include <QDomDocument>
 
+#include <glib.h>
+
 // This is a mock, specifically to test the LauncherModel
 class MockApp: public unity::shell::application::ApplicationInfoInterface
 {
@@ -47,8 +49,6 @@ public:
     QString name() const override { return "mock"; }
     QString comment() const override { return "this is a mock"; }
     QUrl icon() const override { return QUrl(); }
-    ApplicationInfoInterface::Stage stage() const override { return ApplicationInfoInterface::MainStage; }
-    void setStage(ApplicationInfoInterface::Stage) override {}
     ApplicationInfoInterface::State state() const override { return ApplicationInfoInterface::Running; }
     bool focused() const override { return m_focused; }
     QString splashTitle() const override { return QString(); }
@@ -64,12 +64,17 @@ public:
     void setExemptFromLifecycle(bool) override {}
     QSize initialSurfaceSize() const override { return QSize(); }
     void setInitialSurfaceSize(const QSize &) override {}
+    MirSurfaceListInterface* surfaceList() const override { return nullptr; }
+    MirSurfaceListInterface* promptSurfaceList() const override { return nullptr; }
+    int surfaceCount() const override { return m_surfaceCount; }
+    void setSurfaceCount(int count) { m_surfaceCount = count; Q_EMIT surfaceCountChanged(count); }
 
     // Methods used for mocking (not in the interface)
     void setFocused(bool focused) { m_focused = focused; Q_EMIT focusedChanged(focused); }
 private:
     QString m_appId;
     bool m_focused;
+    int m_surfaceCount = 0;
 };
 
 // This is a mock, specifically to test the LauncherModel
@@ -106,7 +111,7 @@ public:
         }
         return false;
     }
-    bool focusApplication(const QString &appId) override {
+    bool focusApplication(const QString &appId) {
         Q_FOREACH(MockApp* app, m_list) {
             app->setFocused(app->appId() == appId);
         }
@@ -114,8 +119,6 @@ public:
         return true;
     }
 
-    void unfocusCurrentApplication() override { }
-
     void addApplication(MockApp *app) {
         beginInsertRows(QModelIndex(), count(), count());
         m_list.append(app);
@@ -142,7 +145,7 @@ private:
 
     QList<QVariantMap> getASConfig() {
         AccountsServiceDBusAdaptor *as = launcherModel->m_asAdapter->m_accounts;
-        QDBusReply<QVariant> reply = as->getUserPropertyAsync(QString(qgetenv("USER")),
+        QDBusReply<QVariant> reply = as->getUserPropertyAsync(QString::fromUtf8(g_get_user_name()),
                                                               "com.canonical.unity.AccountsService",
                                                               "LauncherItems");
         return qdbus_cast<QList<QVariantMap>>(reply.value().value<QDBusArgument>());
@@ -167,7 +170,7 @@ private Q_SLOTS:
                                          QStringLiteral("/org/freedesktop/Accounts"),
                                          QStringLiteral("org.freedesktop.Accounts"));
         QDBusReply<bool> addReply = accountsInterface.call(QStringLiteral("AddUser"),
-                                                           QString(qgetenv("USER")));
+                                                           QString::fromUtf8(g_get_user_name()));
         QVERIFY(addReply.isValid());
         QCOMPARE(addReply.value(), true);
 
@@ -193,7 +196,7 @@ private Q_SLOTS:
                                          QStringLiteral("/org/freedesktop/Accounts"),
                                          QStringLiteral("org.freedesktop.Accounts"));
         QDBusReply<bool> removeReply = accountsInterface.call(QStringLiteral("RemoveUser"),
-                                                              QString(qgetenv("USER")));
+                                                              QString::fromUtf8(g_get_user_name()));
         QVERIFY(removeReply.isValid());
         QCOMPARE(removeReply.value(), true);
     }
@@ -486,6 +489,31 @@ private Q_SLOTS:
 
         // Finally check, that the change to "count" implicitly also set the alerting-state to true
         QVERIFY(launcherModel->get(index)->alerting() == true);
+
+        // Check if the launcher emitted the changed signals
+        QCOMPARE(spy.count(), 2);
+
+        QVariantList countEmissionArgs = spy.takeFirst();
+        QCOMPARE(countEmissionArgs.at(0).value<QModelIndex>().row(), index);
+        QCOMPARE(countEmissionArgs.at(1).value<QModelIndex>().row(), index);
+        QVector<int> roles = countEmissionArgs.at(2).value<QVector<int> >();
+        QCOMPARE(roles.first(), (int)LauncherModel::RoleCount);
+
+        QVariantList countVisibleEmissionArgs = spy.takeFirst();
+        QCOMPARE(countVisibleEmissionArgs.at(0).value<QModelIndex>().row(), index);
+        QCOMPARE(countVisibleEmissionArgs.at(1).value<QModelIndex>().row(), index);
+        roles = countVisibleEmissionArgs.at(2).value<QVector<int> >();
+        QVERIFY(roles.contains(LauncherModel::RoleCountVisible));
+        QVERIFY(roles.contains(LauncherModel::RoleAlerting));
+
+        // Check if the values match
+        QCOMPARE(launcherModel->get(index)->countVisible(), true);
+        QCOMPARE(launcherModel->get(index)->count(), 55);
+        QCOMPARE(launcherModel->get(index)->alerting(), true);
+
+        // Focus the app, make sure the alert gets cleared
+        appManager->focusApplication("abs-icon");
+        QVERIFY(launcherModel->get(index)->alerting() == false);
     }
 
     void testCountEmblemAddsRemovesItem_data() {
@@ -557,6 +585,10 @@ private Q_SLOTS:
 
         // Check that the alerting-status is now true
         QVERIFY(launcherModel->get(index)->alerting() == true);
+
+        // Focus the app, make sure the alert gets cleared
+        appManager->focusApplication("abs-icon");
+        QVERIFY(launcherModel->get(index)->alerting() == false);
     }
 
     void testRefreshAfterDeletedDesktopFiles_data() {
@@ -690,6 +722,15 @@ private Q_SLOTS:
         QCOMPARE(getASConfig().at(index).value("countVisible").toBool(), true);
         QCOMPARE(getASConfig().at(index).value("count").toInt(), 55);
     }
+
+    void testSurfaceCountUpdates() {
+        QString appId = launcherModel->get(0)->appId();
+
+        QCOMPARE(launcherModel->get(0)->surfaceCount(), 0);
+        MockApp *app = qobject_cast<MockApp*>(appManager->findApplication(appId));
+        app->setSurfaceCount(1);
+        QCOMPARE(launcherModel->get(0)->surfaceCount(), 1);
+    }
 };
 
 QTEST_GUILESS_MAIN(LauncherModelTest)
